package com.example.backend.users.service;

import java.util.List;
import java.util.Optional;

import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.util.stream.StreamSupport;

import com.example.backend.core.configurations.Constants;
import com.example.backend.core.errors.NotFoundException;
import com.example.backend.core.security.UserPrincipal;
import com.example.backend.users.model.UserEntity;
import com.example.backend.users.model.UserRole;
import com.example.backend.users.repository.UserRepository;

@Service
public class UserService implements UserDetailsService {
    private final UserRepository repository;
    private final PasswordEncoder passwordEncoder;

    public UserService(UserRepository repository, PasswordEncoder passwordEncoder) {
        this.repository = repository;
        this.passwordEncoder = passwordEncoder;
    }

    private void checkLogin(Integer id, String login) {
        final Optional<UserEntity> existsUser = repository.findByLoginIgnoreCase(login);
        if (existsUser.isPresent() && !existsUser.get().getId().equals(id)) {
            throw new IllegalArgumentException(
                    String.format("User with login %s is already exists", login));
        }
    }

    @Transactional(readOnly = true)
    public List<UserEntity> getAll() {
        return StreamSupport.stream(repository.findAll().spliterator(), false).toList();
    }

    @Transactional(readOnly = true)
    public UserEntity get(Integer id) {
        return repository.findById(id).orElseThrow(() -> new NotFoundException(id));
    }

    @Transactional(readOnly = true)
    private UserEntity getByLogin(String username) {
        return repository.findByLoginIgnoreCase(username)
                .orElseThrow(() -> new IllegalArgumentException("Такого логина нет... хде вы его взяли?"));
    }

    @Transactional
    public UserEntity create(UserEntity entity) {
        if (entity == null) {
            throw new IllegalArgumentException("Entity is null");
        }
        checkLogin(null, entity.getLogin());
        final String password = Optional.ofNullable(entity.getPassword()).orElse("");
        entity.setPassword(
                passwordEncoder.encode(StringUtils.hasText(password.strip()) ? password : Constants.DEFAULT_PASSWORD));
        entity.setRole(Optional.ofNullable(entity.getRole()).orElse(UserRole.USER));
        repository.save(entity);
        return repository.save(entity);
    }

    @Transactional
    public UserEntity update(Integer id, UserEntity entity) {
        final UserEntity existsentity = get(id);
        checkLogin(id, entity.getLogin());
        existsentity.setLogin(entity.getLogin());
        existsentity.setPassword(entity.getPassword());
        existsentity.setRole(entity.getRole());
        repository.save(existsentity);
        return existsentity;
    }

    @Transactional
    public UserEntity delete(Integer id) {
        final UserEntity existsentity = get(id);
        repository.delete(existsentity);
        return existsentity;
    }

    @Override
    @Transactional(readOnly = true)
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
        final UserEntity existUser = getByLogin(username);
        return new UserPrincipal(existUser);
    }
}
